package cz.muni.fi.pa165.core.metrics;

import cz.muni.fi.pa165.model.dto.metrics.MetricsCreateDto;
import cz.muni.fi.pa165.model.dto.metrics.MetricsDto;
import cz.muni.fi.pa165.model.dto.metrics.MetricsUpdateDto;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/api/metric")
@Tag(name = "metric", description = "Metric API")
public class MetricsController {
	private MetricsFacade metricsFacade;

	@Autowired
	public MetricsController(MetricsFacade metricsFacade) {
		this.metricsFacade = metricsFacade;
	}

	@Operation(summary = "Find all metrics (pageable)", description = "Returns a pageable list of all metrics.")
	@GetMapping("/pageable")
	@ApiResponse(responseCode = "200", description = "Successfully retrieved the pageable list of metrics.")
	public List<MetricsDto> findAllPageable(Pageable pageable) {
		return metricsFacade.findAllPageable(pageable);
	}

	@Operation(summary = "Find all metrics", description = "Returns a list of all metrics.")
	@GetMapping
	@ApiResponse(responseCode = "200", description = "Successfully retrieved the list of metrics.")
	public List<MetricsDto> findAll() {
		return metricsFacade.findAll();
	}

	@Operation(summary = "Find metric by ID", description = "Returns the metric with the specified ID.")
	@GetMapping("/{id}")
	@ApiResponse(responseCode = "200", description = "Successfully retrieved the metric.")
	@ApiResponse(responseCode = "404", description = "Metric not found.")
	public MetricsDto findById(@PathVariable @Parameter(description = "The ID of the metric to retrieve.") String id) {
		return metricsFacade.findById(id);
	}

	@Operation(summary = "Create metric", description = "Creates a new metric.")
	@PostMapping
	@ApiResponse(responseCode = "201", description = "Successfully created a new metric.")
	public MetricsDto create(@RequestBody @Valid MetricsCreateDto metricsCreateDto) {
		return metricsFacade.create(metricsCreateDto);
	}

	@Operation(summary = "Update metric", description = "Updates the metric with the specified ID.")
	@PutMapping("/{id}")
	@ApiResponse(responseCode = "200", description = "Successfully updated the metric.")
	@ApiResponse(responseCode = "404", description = "Metric not found.")
	public MetricsDto updateById(@PathVariable @Parameter(description = "The ID of the metric to update.") String id,
								 @RequestBody @Valid MetricsUpdateDto metricsUpdateDto) {
		return metricsFacade.updateById(metricsUpdateDto, id);
	}

	@Operation(summary = "Delete metric", description = "Deletes the metric with the specified ID.")
	@DeleteMapping("/{id}")
	@ApiResponse(responseCode = "200", description = "Successfully deleted the metric.")
	@ApiResponse(responseCode = "404", description = "Metric not found.")
	public MetricsDto deleteById(@PathVariable @Parameter(description = "The ID of the metric to delete.") String id) {
		return metricsFacade.deleteById(id);
	}
}
