package cz.muni.fi.pa165.microservice4.electricityprices;

import jakarta.annotation.PostConstruct;
import org.springframework.stereotype.Service;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.List;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.CopyOnWriteArrayList;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;

@Service
public class ElectricityPriceServiceImpl implements ElectricityPriceService {

  // to be stored in database in the future
  private final List<ElectricityPrice> list = new CopyOnWriteArrayList<>();

  @PostConstruct
  public void init() {
    list.add(
        ElectricityPrice.builder()
            .id("1")
            .companyId("1")
            .priceHighTariff(300.0)
            .priceLowTariff(200.0)
            .build());
  }

  @Override
  @Transactional(readOnly = true)
  public double getElectricityPrice(String companyId) {
    return list.stream()
            .filter(price -> Objects.equals(companyId, price.getCompanyId()))
            .findFirst()
            .map(this::getPriceAcordingToDate)
            .orElseThrow(() -> new NoElectricityPriceForCompanyException("No electricity price information for that company."));
}

  private double getPriceAcordingToDate(ElectricityPrice price) {
    LocalTime localTime = LocalTime.now();
    //weekend prices
    if(LocalDate.now().getDayOfWeek() == DayOfWeek.SATURDAY || LocalDate.now().getDayOfWeek() == DayOfWeek.SUNDAY)
      return price.getPriceLowTariff();
    //peak hours
    if(localTime.getHour() > 7 && localTime.getHour() < 21)
      return price.getPriceHighTariff();
    //off-peak hours
    return price.getPriceLowTariff();
  }

  @Override
  @Transactional(readOnly = true)
  public List<ElectricityPrice> getAllElectricityPrice() {
    return list;
  }

  public ElectricityPrice setElectricityPrice(ElectricityPriceSetRequest req) {
    for (ElectricityPrice price:
         list) {
      if(Objects.equals(price.getCompanyId(), req.getCompanyId())) {
        price.setPriceLowTariff(req.getPriceLowTariff());
        price.setPriceHighTariff(req.getPriceHighTariff());
        return price;
      }
    }
    ElectricityPrice newPrice = new ElectricityPrice(UUID.randomUUID().toString(), req.getCompanyId(), req.getPriceHighTariff(), req.getPriceLowTariff());
    list.add(newPrice);
    return newPrice;
  }

  @Override
  @ExceptionHandler
  public String deleteElectricityPriceForCompany(String companyId) {
    for (ElectricityPrice price:
            list) {
      if(Objects.equals(price.getCompanyId(), companyId)) {
        list.remove(price);
        return "deleted electricity prices for company with companyId " + companyId;
      }
    }
    throw new NoElectricityPriceForCompanyException("Did not delete the electricity prices - company not found.");
  }


}
